-- keep the filename of this file the same as the folder that contains it
-- global variables accessible by any module we import
Dt = 0
Time = 0

local settings = require('settings.first')
local gearbox = require('gearbox.first')
local sectors = require('sectors.first')
local tires = require('tires.first')
local leaderboard = require('leaderboard.first')
local player_data = require('common.players')
local map = require('map.first')
local time = require('time.first')
local pctime = require('pctime.first')
local radar = require('radar.first')
local pedals = require('pedals.first')
local deltabar = require('deltabar.first')

local init = false

local session_time = -999999
local err_count = 1
local function call_protected_function(func)
    xpcall(func, function(err)
        ac.debug("ERROR: " .. err_count, err .. "\n" .. debug.traceback())
        err_count = err_count + 1
    end)
end

local function session_start(session_index, restarted)
    call_protected_function(player_data.on_session_start)
    call_protected_function(sectors.on_session_start)
    call_protected_function(deltabar.on_session_start) -- before leaderboard so he gets the correct data
    call_protected_function(leaderboard.on_session_start)
    call_protected_function(deltabar.on_session_start)
    call_protected_function(tires.on_session_start)
end

local function on_game_close(nothing)
    call_protected_function(deltabar.on_game_close)
    call_protected_function(gearbox.on_game_close)
end

function script.update(dt)
    err_count = 1
    Dt = dt
    Time = Time + Dt
    
    local sim_info = ac.getSim()
    if init == false then
        init = true
        ac.onRelease(on_game_close, nil)
        if sim_info.isOnlineRace == false then
            ac.onSessionStart(session_start)
        end
        call_protected_function(settings.init)
        call_protected_function(player_data.init)
        call_protected_function(sectors.init)
        call_protected_function(tires.init)
        call_protected_function(map.init)
        call_protected_function(deltabar.init) -- before leaderboard so we have proper data on players' laps
        call_protected_function(leaderboard.init)
        call_protected_function(gearbox.init)
        call_protected_function(pedals.init)
    end
    
    if sim_info.isOnlineRace then
        if sim_info.currentSessionTime < session_time then
            session_start(-1, -1)
        end
    end
    session_time = sim_info.currentSessionTime
    
    call_protected_function(player_data.update)
    call_protected_function(tires.update)
    call_protected_function(sectors.update)
    call_protected_function(leaderboard.update)
    call_protected_function(map.update)
    call_protected_function(deltabar.update)
    call_protected_function(pedals.update)
end

function maptimeMain(dt) call_protected_function(time.main) end
function pctimeMain(dt) call_protected_function(pctime.main) end
function settingsMain(dt) call_protected_function(settings.main) end

function pedalsMain(dt) call_protected_function(pedals.main) end

function deltabarMain(dt) call_protected_function(deltabar.main) end
function deltabarShow(dt) call_protected_function(deltabar.on_open) end
function deltabarHide(dt) call_protected_function(deltabar.on_close) end

function radarMain(dt) call_protected_function(radar.main) end
function radarShow(dt) call_protected_function(radar.on_open) end
function radarHide(dt) call_protected_function(radar.on_close) end

function tiresMain(dt) call_protected_function(tires.main) end
function tyresShow(dt) call_protected_function(tires.on_open) end
function tyresHide(dt) call_protected_function(tires.on_close) end

function gearboxMain(dt) call_protected_function(gearbox.main) end
function gearboxShow(dt) call_protected_function(gearbox.on_open) end
function gearboxHide(dt) call_protected_function(gearbox.on_close) end

function sectorsMain(dt) call_protected_function(sectors.main) end
function sectorsShow(dt) call_protected_function(sectors.on_open) end
function sectorsHide(dt) call_protected_function(sectors.on_close) end

function leaderboardMain(dt) call_protected_function(leaderboard.main) end
function leaderboardShow(dt) call_protected_function(leaderboard.on_open) end
function leaderboardHide(dt) call_protected_function(leaderboard.on_close) end

function mapMain(dt) call_protected_function(map.main) end
function mapShow(dt) call_protected_function(map.on_open) end
function mapHide(dt) call_protected_function(map.on_close) end

